
INCLUDE "gbhw.inc"
        INCLUDE "hardware.inc"

;        INCLUDE "ibmpc1.inc"
;		Section "TimerInt",HOME[$50]
;    		jp Timer
Section "VblankInt",HOME[$40]
    push af
    ld a,1
    ld [_IsItVBlank],a
    pop af
    reti
    
        SECTION "Org $100",HOME[$100]

        nop
        jp      begin

;  To include the standard ROM header information we
; can just use the macro ROM_HEADER. We defined this macro
; earlier when we INCLUDEd "gbhw.inc".
;
;  The ROM_NOMBC just suggests to the complier that we are
; not using a Memory Bank Controller because we don't need one
; since our ROM won't be larger than 32K bytes.
;
;  Next we indicate the cart ROM size and then the cart RAM size.
; We don't need any cart RAM for this program so we set this to 0K.

        ROM_HEADER      ROM_NOMBC, ROM_SIZE_32KBYTE, RAM_SIZE_0KBYTE

; Next we need to include some code for doing
; RAM copy, RAM fill, etc.
		INCLUDE "ingametiledta.inc"
		INCLUDE "ingamemap.inc"
        INCLUDE "memory.asm"
        GLOBAL TilePals
        
; Joypad code
		GLOBAL pad_Read

;  Next, let's actually include font tile data into the ROM
; that we are building. We do this by invoking the chr_IBMPC1
; macro that was defined earlier when we INCLUDEd "ibmpc1.inc".
;
;  The 1 & 8 parameters define that we want to include the
; whole IBM-PC font set and not just parts of it.
;
;  Right before invoking this macro we define the label
; TileData. Whenever a label is defined with nothing following
; it (except possibly a colon) it is given the value of the
; current ROM location.
;  As a result, TileData now has a memory location value that
; is the same as the first byte of the font data that we are
; including. We shall use the label TileData as a "handle" or
; "reference" for locating our font data.



;  The NOP and then JP located at $100 in ROM are executed
; which causes the the following code to be executed next.

begin:

; First, it's a good idea to Disable Interrupts
; using the following command. We won't be using
; interrupts in this example so we can leave them off.

        di

;  Next, we should initialize our stack pointer. The
; stack pointer holds return addresses (among other things)
; when we use the CALL command so the stack is important to us.
;
;  The CALL command is similar to the GOSUB command
; in the BASIC language and it is similar to executing
; a procedure in the C & PASCAL languages.
;
; We shall set the stack to the top of high ram + 1.
; (For more information on the Stack Pointer register
; read the section on Registers.)
;

        ld      sp,$ffff

;  Next we shall turn the Liquid Crystal Display (LCD)
; off so that we can copy data to video RAM. We can
; copy data to video RAM while the LCD is on but it
; is a little more difficult to do and takes a little
; bit longer. Video RAM is not always available for
; reading or writing when the LCD is on so it is
; easier to write to video RAM with the screen off.
; ( In future examples, writing to video RAM while the
; screen is on will be covered.)
;
;  To turn off the LCD we do a CALL to the StopLCD
; subroutine at the bottom of this file. The reason
; we use a subroutine is because it takes more than
; just writing to a memory location to turn the
; LCD display off. The LCD display should be in
; Vertical Blank (or VBlank) before we turn the display
; off. Weird effects can occur if you don't wait until
; VBlank to do this and code written for the Super
; GameBoy won't work sometimes you try to turn off
; the LCD outside of VBlank.
;
;  The StopLCD routine is terminated by a RET command.
; The RET command is similar to the RETURN command in
; BASIC.

        call    StopLCD

;  Here we are going to setup the background tile
; palette so that the tiles appear in the proper
; shades of grey.
;
;  To do this, we need to write the value $e4 to the
; memory location $ff47. In the 'gbhw.inc' file we
; INCLUDEd there is a definition that rBGP=$ff47 so
; we can use the rGBP label to do this
;
;  The first instruction loads the value $e4 into the
; 8-bit register A and the second instruction writes
; the value of register A to memory location $ff47.

        ld      a,$e4
        ld      [rBGP],a        ; Setup the default background palette

;  Here we are setting the X/Y scroll registers
; for the tile background to 0 so that we can see
; the upper left corner of the tile background.
;
;  Think of the tile background RAM (which we usually call
; the tile map RAM) as a large canvas. We draw on this
; 'canvas' using 'paints' which consist of tiles and
; sprites (we will cover sprites in another example.)
;
;  We set the scroll registers to 0 so that we can
; view the upper left corner of the 'canvas'.

        ld      a,0
        ld      [rSCX],a
        ld      [rSCY],a

        ld      hl,InGameTileData
        ld      de,$8000
        ld      bc,8*256        ; length (8 bytes per tile) x (256 tiles)
        call    mem_Copy	    ; Copy tile data to memory

		ld 		a,$01
        ld		[$FF4F],a
        
        ld      hl,DefaultMapPLN1;Draw title
        ld      de,$9800
        ld      bc,9*32
        call    mem_Copy
        
        ld 		a,$00
        ld		[$FF4F],a

        ld      hl,DefaultMapPLN0;Draw title
        ld      de,$9800
        ld      bc,9*32
        call    mem_Copy        
        
        ld      hl,TilePals
        call    SetAllCGB_BGP
        
        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ld      [rLCDC],a       ; Turn screen on
		
		ld 		a,$15
		ld		[_TimeRemaining],a
		
		ld 		a,$FE
		ld		[_Timer],a	
		
.gameloop:
		call 	Timer
		call 	UpdateTR
		call 	joypad
        jp      .gameloop

.you_shouldnt_be_here:
		jp begin
		ret
		
	
; UpdateTimeRemaining function
; Updates timer on screen
UpdateTR:
		; get the amount of time left
		ld a,0
		ld b,a
		ld c,a
		ld d,a
		ld a, [_TimeRemaining]
		;ld a,$14
		ld c,a
		;a = 0
		ld a, 0
		;While c > 0
.whileloop:
		push af
		ld a,c
		cp 0
		jp z,.wend
		jp c,.wend
		pop af
		
		    ;b = 0
		    push af
		    ld a,0
		    ld b,a
		    pop af
		    
		    ;For counter = 0 To 9
		    push af
		    ld a,0
.forloop:
			cp 9
			jp nc,.next
			
		        ;c = c - 1
		        push af
		        dec c
		        pop af

		        ;If c >= 0 Then
		        ld d,a
		        ld a,c
		        cp 0
		        jp c,.noif
		        
			        ;b = b + 1
			        
		            inc b
.noif:
				ld a,d
		    ;Next
		    inc a
		    jp .forloop
.next:
			pop af
			
			;If c >= 0 Then
		    push af
	        ld a,c
	        cp 0
	        jp c,.noif2
			pop af
		        ;a = a + 1
		        inc a
		    ;End If
.noif2:
		jp .whileloop
.wend:
		pop af
		; All right! We're done with our place-val stuff (I knew that lesson in school would come in handy ^_^)
		; wait for VBL
		call	WaitVBL
		lcd_WaitVRAM
		; add 14 to A...
		add 14
		; load it in.
		ld [$9800 + 6], a
		lcd_WaitVRAM

		ld a, b
		; add 14 to A...
		add 14
		; load it in.
		ld [$9800 + 7], a
		ret		

; Timer Function
; Updates Timer and also TimeRemaining if necessary.

Timer:
		; First order of business: Is the timer zero yet?
		; Get ready, don't destroy anything!
		push	af
		; Get the timer ready!
		ld		a, [_Timer]
		; Compare it
		cp		0
		; Skip or no skip?
		jp		nz,.skipreset
		; OK, no skip...
		; Reset the timer
		ld		a, $FE
		ld		[_Timer], a
		ld		a, [_TimeRemaining]
		cp		0
		jp 		nz,.minustr
		ld		a, $fe
		ld		[_TimeRemaining], a
		jp		.endsub
.minustr:
		sub 1
		ld		[_TimeRemaining], a
		jp		.endsub
.skipreset:
		ld		a, [_Timer]
		sub 1
		ld		[_Timer], a
		jp .endsub
.endsub:
		pop af
		ret
; Joypad Function
		
joypad:
		call pad_Read
		bit 4,a
		jp nz,.right
		bit 5,a
		jp nz,.left
		bit 6,a
		jp nz,.up
		bit 7,a
		jp nz,.down
		ret
.left:
		ld a,[rSCX]
		add 1
		ld [rSCX],a
		ret
.right:
		ld a,[rSCX]
		sub 1
		ld [rSCX],a
		ret
.up:
		ld a,[rSCY]
		add 1
		ld [rSCY],a
		ret
.down:
		ld a,[rSCY]
		sub 1
		ld [rSCY],a
		ret
				
WaitVBL:
        ld      a,[rLY]
        cp      0               ; Is display on scan line 145 yet?
        jr      nz,WaitVBL      ; no, keep waiting
        ret


; *** Turn off the LCD display ***

StopLCD:
        ld      a,[rLCDC]
        rlca                    ; Put the high bit of LCDC into the Carry flag
        ret     nc              ; Screen is off already. Exit.

; Loop until we are in VBlank

.wait:
        ld      a,[rLY]
        cp      145             ; Is display on scan line 145 yet?
        jr      nz,.wait        ; no, keep waiting

; Turn off the LCD

        ld      a,[rLCDC]
        res     7,a             ; Reset bit 7 of LCDC
        ld      [rLCDC],a

        ret




SetSingleCGB_BGP:
        add     a,a             ; a = pal # * 2
        add     a,a             ; a = pal # * 4
        add     a,a             ; a = pal # * 8
        add     a,$80
        ld      [$FF68],a

        ld      bc,$0869        ; b = 8, c = rBCPD
.loop1:
        di
.loop2:
        ld      a,[rSTAT]
        and     2
        jr      nz,.loop2

        ld      a,[hl+]
        ld      [c],a
        ei

        dec     b
        jr      nz,.loop1

        ret

; *** Set all background palettes ***
; Entry: HL = pntr to data for 8 palettes

SetAllCGB_BGP:
        ld      a,$80
        ld      [$FF68],a

        ld      bc,$4069        ; b = 64, c = rBCPD
.loop1:
        di
.loop2:
        ld      a,[rSTAT]
        and     2
        jr      nz,.loop2

        ld      a,[hl+]
        ld      [c],a
        ei

        dec     b
        jr      nz,.loop1

        ret

; *** Set a single object (sprite) palette ***
; Entry: HL = pntr to data for 1 palette
;         A = palette number (0-7)

SetSingleCGB_OBJP:
        add     a,a             ; a = pal # * 2
        add     a,a             ; a = pal # * 4
        add     a,a             ; a = pal # * 8
        add     a,$80
        ld      [$FF6A],a

        ld      bc,$086b        ; b = 8, c = rOCPD
.loop1:
        di
.loop2:
        ld      a,[rSTAT]
        and     2
        jr      nz,.loop2

        ld      a,[hl+]
        ld      [c],a
        ei

        dec     b
        jr      nz,.loop1

        ret

; *** Set all object (sprite) palettes ***
; Entry: HL = pntr to data for 8 palettes

SetAllCGB_OBJP:

        ld      a,$80
        ld      [$FF6A],a

        ld      bc,$406b        ; b = 64, c = rOCPD
.loop1:
        di
.loop2:
        ld      a,[rSTAT]
        and     2
        jr      nz,.loop2

        ld      a,[hl+]
        ld      [c],a
        ei

        dec     b
        jr      nz,.loop1

        ret

;  This RGBSet Macro uses RGB values from 0 to 255.
; Even though this is a greater color range than the
; GBC supports it allows you to directly read color
; values from your favorite paint program using the
; eye dropper tool and drop them directly into your
; program.


RGBSet: MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

; ** Example colo(u)r palettes **

BGPaletteTable:
        RGBSet  255,255,255     ; White         - Palette #1
        RGBSet  191,191,191     ; Light Grey
        RGBSet  127,127,127     ; Dark Grey
        RGBSet  0,0,0           ; Black

        RGBSet  255,0,0         ; Red           - Palette #2
        RGBSet  0,255,0         ; Green
        RGBSet  0,0,255         ; Blue
        RGBSet  0,0,0           ; Black

;     ........

        RGBSet  255,255,0       ; Yellow        - Palette #8
        RGBSet  255,0,255       ; Purple
        RGBSet  127,127,127     ; Dark Grey
        RGBSet  0,0,0           ; Black


OBJPaletteTable:
        RGBSet  0,0,0           ; Transparent (not used, value not important)
        RGBSet  191,191,191     ; Light Grey
        RGBSet  127,127,127     ; Dark Grey
        RGBSet  0,0,0           ; Black

        RGBSet  0,0,0           ; Transparent            - Palette #2
        RGBSet  0,255,0         ; Green
        RGBSet  0,0,255         ; Blue
        RGBSet  0,0,0           ; Black

;     ........

        RGBSet  0,0,0           ; Transparent            - Palette #8
        RGBSet  255,0,255       ; Purple
        RGBSet  127,127,127     ; Dark Grey
        RGBSet  0,0,0           ; Black

; ** end of file **


SECTION "InternalVars",BSS
_Timer:: 			DS 1		;holds time until next second
_TimeRemaining:: 	DS 1		;starts out $14 (20 decimal), holds time remaining
_IsItVBlank::		DS 1		